/*
    This file is part of cpp-ethereum.

    cpp-ethereum is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    cpp-ethereum is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with cpp-ethereum.  If not, see <http://www.gnu.org/licenses/>.
*/
/** @file CommonData.cpp
 * @author Gav Wood <i@gavwood.com>
 * @date 2014
 */

#include "CommonData.h"

#include <stdexcept>

using namespace std;
using namespace dev;

namespace {
int
fromHexChar(char _i) noexcept
{
    if (_i >= '0' && _i <= '9')
        return _i - '0';
    if (_i >= 'a' && _i <= 'f')
        return _i - 'a' + 10;
    if (_i >= 'A' && _i <= 'F')
        return _i - 'A' + 10;
    return -1;
}
} // namespace

bool
dev::isHex(string const& _s) noexcept
{
    auto it = _s.begin();
    if (_s.compare(0, 2, "0x") == 0)
        it += 2;
    return std::all_of(it, _s.end(), [](char c) { return fromHexChar(c) != -1; });
}

bytes
dev::fromHex(std::string const& _s, WhenError _throw)
{
    unsigned s = (_s.size() >= 2 && _s[0] == '0' && _s[1] == 'x') ? 2 : 0;
    std::vector<uint8_t> ret;
    ret.reserve((_s.size() - s + 1) / 2);

    if (_s.size() % 2) {
        int h = fromHexChar(_s[s++]);
        if (h != -1)
            ret.push_back(h);
        else if (_throw == WhenError::Throw)
            throw std::runtime_error("BadHexCharacter");
        else
            return bytes();
    }
    for (unsigned i = s; i < _s.size(); i += 2) {
        int h = fromHexChar(_s[i]);
        int l = fromHexChar(_s[i + 1]);
        if (h != -1 && l != -1)
            ret.push_back((uint8_t)(h * 16 + l));
        else if (_throw == WhenError::Throw)
            throw std::runtime_error("BadHexCharacter");
        else
            return bytes();
    }
    return ret;
}

bytes
dev::asNibbles(bytesConstRef const& _s)
{
    std::vector<uint8_t> ret;
    ret.reserve(_s.size() * 2);
    for (auto i : _s) {
        ret.push_back(i / 16);
        ret.push_back(i % 16);
    }
    return ret;
}
